@@ -33,6 +33,7 @@
 #include "ctkhelp.h"
 #include "ctkcolorcontrols.h"
 #include "ctkdropdownmenu.h"
+#include "ctkutils.h"
 
 /* function prototypes */
 static void
@@ -56,9 +57,9 @@ static void color_space_menu_changed(GtkWidget *widget,
 static void color_range_menu_changed(GtkWidget *widget,
                                      gpointer user_data);
 
-static void color_control_update_received(GtkObject *object, gpointer arg1,
+static void color_control_update_received(GObject *object, CtrlEvent *event,
                                           gpointer user_data);
-static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls);
+static void setup_color_range_dropdown(CtkColorControls *ctk_color_controls);
 static
 void post_color_range_update(CtkColorControls *ctk_color_controls,
                              gint color_range);
@@ -136,7 +137,7 @@ static void ctk_color_controls_finalize(GObject *object)
 
 
 
-GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
+GtkWidget* ctk_color_controls_new(CtrlTarget *ctrl_target,
                                   CtkConfig *ctk_config,
                                   CtkEvent *ctk_event,
                                   GtkWidget *reset_button,
@@ -152,10 +153,10 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     gint i;
 
     /* check if color configuration is supported */
-    ret1 = NvCtrlGetValidAttributeValues(handle,
+    ret1 = NvCtrlGetValidAttributeValues(ctrl_target,
                                          NV_CTRL_COLOR_SPACE,
                                          &valid1);
-    ret2 = NvCtrlGetValidAttributeValues(handle,
+    ret2 = NvCtrlGetValidAttributeValues(ctrl_target,
                                          NV_CTRL_COLOR_RANGE,
                                          &valid2);
 
@@ -170,7 +171,7 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     }
 
     ctk_color_controls = CTK_COLOR_CONTROLS(object);
-    ctk_color_controls->handle = handle;
+    ctk_color_controls->ctrl_target = ctrl_target;
     ctk_color_controls->ctk_config = ctk_config;
     ctk_color_controls->ctk_event = ctk_event;
     ctk_color_controls->reset_button = reset_button;
@@ -198,7 +199,7 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
 
     /* dropdown list for color space */
     menu = (CtkDropDownMenu *)
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
 
     for (i = 0; i < ctk_color_controls->color_space_table_size; i++) {
         switch (ctk_color_controls->color_space_table[i]) {
@@ -257,7 +258,7 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     /* dropdown list for color range */
     
     ctk_color_controls->color_range_menu =
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
     
     ctk_config_set_tooltip(ctk_config, 
                            ctk_color_controls->color_range_menu,
@@ -312,14 +313,14 @@ static void setup_reset_button(CtkColorControls *ctk_color_controls)
     gint val;
     CtkDropDownMenu *color_space_menu, *color_range_menu;
 
-    if (!GTK_WIDGET_SENSITIVE(ctk_color_controls->color_controls_box)) {
+    if (!ctk_widget_get_sensitive(ctk_color_controls->color_controls_box)) {
         /* Nothing is available, don't bother enabling the reset button yet. */
         return;
     }
 
     /* The color space menu is always available */
     color_space_menu = CTK_DROP_DOWN_MENU(ctk_color_controls->color_space_menu);
-    history = color_space_menu->current_selected_item;
+    history = ctk_drop_down_menu_get_current_value(color_space_menu);
 
     val = ctk_color_controls->color_space_table[history];
     if (val != NV_CTRL_COLOR_SPACE_RGB) {
@@ -327,10 +328,10 @@ static void setup_reset_button(CtkColorControls *ctk_color_controls)
     }
 
     /* Color range is dependent on the color space */
-    if (GTK_WIDGET_SENSITIVE(ctk_color_controls->color_range_menu)) {
+    if (ctk_widget_get_sensitive(ctk_color_controls->color_range_menu)) {
         color_range_menu = 
             CTK_DROP_DOWN_MENU(ctk_color_controls->color_range_menu);
-        history = color_range_menu->current_selected_item; 
+        history = ctk_drop_down_menu_get_current_value(color_range_menu);
         val = ctk_color_controls->color_range_table[history];
         if (val != NV_CTRL_COLOR_RANGE_FULL) {
             goto enable;
@@ -362,7 +363,7 @@ void ctk_color_controls_setup(CtkColorControls *ctk_color_controls)
     /* color space */
     if (!update_color_space_menu_info(ctk_color_controls)) {
         gtk_widget_set_sensitive(ctk_color_controls->color_controls_box, FALSE);
-        gtk_widget_hide_all(ctk_color_controls->color_controls_box);
+        gtk_widget_hide(ctk_color_controls->color_controls_box);
     }
 
     setup_reset_button(ctk_color_controls);
@@ -372,11 +373,12 @@ void ctk_color_controls_setup(CtkColorControls *ctk_color_controls)
 
 static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_controls)
 {
+    CtrlTarget *ctrl_target = ctk_color_controls->ctrl_target;
     gint color_space = NV_CTRL_COLOR_SPACE_RGB;
 
     /* color space */
     if (NvCtrlSuccess !=
-        NvCtrlGetAttribute(ctk_color_controls->handle,
+        NvCtrlGetAttribute(ctrl_target,
                            NV_CTRL_COLOR_SPACE,
                            &color_space)) {
         return FALSE;
@@ -399,11 +401,7 @@ static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_control
          (gpointer) ctk_color_controls);
 
     /* dynamically regenerate color range dropdown */
-    if (!setup_color_range_dropdown(ctk_color_controls)) {
-        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, FALSE);
-    } else {
-        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, TRUE);
-    }
+    setup_color_range_dropdown(ctk_color_controls);
 
     return TRUE;
 
@@ -447,13 +445,14 @@ static void color_range_menu_changed(GtkWidget *widget,
 {
     CtkColorControls *ctk_color_controls =
         CTK_COLOR_CONTROLS(user_data);
+    CtrlTarget *ctrl_target = ctk_color_controls->ctrl_target;
     CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
     gint history, color_range = NV_CTRL_COLOR_RANGE_FULL;
 
     history = ctk_drop_down_menu_get_current_value(menu);
     color_range = ctk_color_controls->color_range_table[history];
 
-    NvCtrlSetAttribute(ctk_color_controls->handle,
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_COLOR_RANGE,
                        color_range);
 
@@ -468,6 +467,7 @@ static void color_space_menu_changed(GtkWidget *widget,
 {
     CtkColorControls *ctk_color_controls =
         CTK_COLOR_CONTROLS(user_data);
+    CtrlTarget *ctrl_target = ctk_color_controls->ctrl_target;
     CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
     gint history, color_space = NV_CTRL_COLOR_SPACE_RGB;
     
@@ -475,15 +475,16 @@ static void color_space_menu_changed(GtkWidget *widget,
 
     color_space = ctk_color_controls->color_space_table[history];
 
-    NvCtrlSetAttribute(ctk_color_controls->handle,
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_COLOR_SPACE,
                        color_space);
 
     color_space = map_nvctrl_value_to_table(ctk_color_controls,
                                             color_space);
 
-    /* reflecting the change in color space to other widgets & reset button */
-    ctk_color_controls_setup(ctk_color_controls);
+    /* Regenerate the color range dropdown to reflect the new color space */
+    setup_color_range_dropdown(ctk_color_controls);
+
     post_color_space_update(ctk_color_controls, color_space);
 
 } /* color_space_menu_changed() */
@@ -495,15 +496,19 @@ static void color_space_menu_changed(GtkWidget *widget,
  */
 void ctk_color_controls_reset(CtkColorControls *ctk_color_controls)
 {
+    CtrlTarget *ctrl_target;
+
     if (!ctk_color_controls) {
         return;
     }
 
-    NvCtrlSetAttribute(ctk_color_controls->handle,
+    ctrl_target = ctk_color_controls->ctrl_target;
+
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_COLOR_SPACE,
                        NV_CTRL_COLOR_SPACE_RGB);
 
-    NvCtrlSetAttribute(ctk_color_controls->handle,
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_COLOR_RANGE,
                        NV_CTRL_COLOR_RANGE_FULL);
 
@@ -538,20 +543,26 @@ void add_color_controls_help(CtkColorControls *ctk_color_controls,
  * we should update the GUI to reflect the current color range
  * and color space.
  */
-static void color_control_update_received(GtkObject *object, gpointer arg1,
+static void color_control_update_received(GObject *object,
+                                          CtrlEvent *event,
                                           gpointer user_data)
 {
     CtkColorControls *ctk_object = CTK_COLOR_CONTROLS(user_data);
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
+
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
 
     ctk_color_controls_setup(ctk_object);
 
     /* update status bar message */
-    switch (event_struct->attribute) {
+    switch (event->int_attr.attribute) {
     case NV_CTRL_COLOR_RANGE:
-        post_color_range_update(ctk_object, event_struct->value); break;
+        post_color_range_update(ctk_object, event->int_attr.value);
+        break;
     case NV_CTRL_COLOR_SPACE:
-        post_color_space_update(ctk_object, event_struct->value); break;
+        post_color_space_update(ctk_object, event->int_attr.value);
+        break;
     }
 } /* color_control_update_received()  */
 
@@ -600,23 +611,25 @@ static gboolean build_color_space_table(CtkColorControls *ctk_color_controls,
  * setup_color_range_dropdown() - dynamically generate dropdown list for
  * color range depending on selected color space.
  */
-static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
+static void setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
 {
+    CtrlTarget *ctrl_target = ctk_color_controls->ctrl_target;
     gint i, n = 0, color_range_count = 0;
     gint mask, val;
+    gboolean enable = FALSE;
     ReturnStatus ret;
     NVCTRLAttributeValidValuesRec valid;
     CtkDropDownMenu *d;
 
-    ret = NvCtrlGetValidAttributeValues(ctk_color_controls->handle,
+    ret = NvCtrlGetValidAttributeValues(ctrl_target,
                                         NV_CTRL_COLOR_RANGE,
                                         &valid);
     if (ret != NvCtrlSuccess) {
-        return FALSE;
+        goto done;
     }
 
     if (valid.type != ATTRIBUTE_TYPE_INT_BITS) {
-        return FALSE;
+        goto done;
     }
     mask = valid.u.bits.ints;
     /* count no. of supported color space */
@@ -633,7 +646,7 @@ static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
     ctk_color_controls->color_range_table =
         calloc(color_range_count, sizeof(ctk_color_controls->color_range_table[0]));
     if (!ctk_color_controls->color_range_table) {
-        return FALSE;
+        goto done;
     }
 
     for (i = 0, n = 0; n < ctk_color_controls->color_range_table_size; i++) {
@@ -667,7 +680,7 @@ static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
 
     /* color range */
     if (NvCtrlSuccess !=
-        NvCtrlGetAttribute(ctk_color_controls->handle,
+        NvCtrlGetAttribute(ctrl_target,
                            NV_CTRL_COLOR_RANGE,
                            &val)) {
         val = NV_CTRL_COLOR_RANGE_FULL;
@@ -682,11 +695,12 @@ static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
 
     /* If dropdown only has one item, disable it */
     if (ctk_color_controls->color_range_table_size <= 1) {
-        return FALSE;
+        goto done;
     }
+    enable = TRUE;
 
-    return TRUE;
-
+done:
+    gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, enable);
 } /* setup_color_range_dropdown() */
 
 
