@@ -4,28 +4,22 @@
  *
  * Copyright (C) 2009 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #ifndef __CTK_ECC_H__
 #define __CTK_ECC_H__
 
-#include "NvCtrlAttributes.h"
 #include "ctkevent.h"
 #include "ctkconfig.h"
 
@@ -56,7 +50,7 @@ struct _CtkEcc
 {
     GtkVBox parent;
 
-    NvCtrlAttributeHandle *handle;
+    CtrlTarget *ctrl_target;
     CtkConfig *ctk_config;
 
     GtkWidget* status;
@@ -69,6 +63,7 @@ struct _CtkEcc
     GtkWidget* configuration_status;
 
     gboolean ecc_enabled;
+    gboolean ecc_configured;
     gboolean ecc_toggle_warning_dlg_shown;
     gboolean dbit_error_available;
     gboolean aggregate_dbit_error_available;
@@ -81,7 +76,7 @@ struct _CtkEccClass
 };
 
 GType          ctk_ecc_get_type    (void) G_GNUC_CONST;
-GtkWidget*     ctk_ecc_new         (NvCtrlAttributeHandle *, CtkConfig *, CtkEvent *);
+GtkWidget*     ctk_ecc_new         (CtrlTarget *, CtkConfig *, CtkEvent *);
 GtkTextBuffer* ctk_ecc_create_help (GtkTextTagTable *, CtkEcc *);
 
 void           ctk_ecc_start_timer (GtkWidget *);
