@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -106,12 +101,12 @@ static void apply_button_clicked            (GtkButton *button,
 static void initialize_csc_dropdown_changed (CtkDropDownMenu *combo,
                                              gpointer user_data);
 
-static void set_apply_button_senstive       (CtkGvoCsc *ctk_gvo_csc);
+static void set_apply_button_sensitive      (CtkGvoCsc *ctk_gvo_csc);
 
 static void apply_csc_values                (CtkGvoCsc *ctk_gvo_csc);
 
-static void gvo_csc_event_received          (GtkObject *object,
-                                             gpointer arg1,
+static void gvo_csc_event_received          (GObject *object,
+                                             CtrlEvent *event,
                                              gpointer user_data);
 
 static GtkWidget *build_opengl_only_msg     (void);
@@ -146,7 +141,7 @@ static const char * __initialize_help =
 "values from the selected standard.";
 
 static const char * __csc_help =
-"The color space conversion matrix defines the paramaters used for "
+"The color space conversion matrix defines the parameters used for "
 "converting the color data.";
 
 static const char * __apply_immediately_help =
@@ -174,6 +169,7 @@ GType ctk_gvo_csc_get_type(void)
             sizeof (CtkGvoCsc),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_gvo_csc_type =
@@ -189,7 +185,7 @@ GType ctk_gvo_csc_get_type(void)
  * ctk_gvo_csc_new() - create a CtkGvoCsc widget
  */
 
-GtkWidget* ctk_gvo_csc_new(NvCtrlAttributeHandle *handle,
+GtkWidget* ctk_gvo_csc_new(CtrlTarget *ctrl_target,
                            CtkConfig *ctk_config,
                            CtkEvent *ctk_event,
                            CtkGvo *gvo_parent)
@@ -209,21 +205,22 @@ GtkWidget* ctk_gvo_csc_new(NvCtrlAttributeHandle *handle,
     float initialCSCMatrix[3][3];
     float initialCSCOffset[3];
     float initialCSCScale[3];
-    
+
     /* retrieve all the NV-CONTROL attributes that we will need */
-    
-    ret = NvCtrlGetGvoColorConversion(handle,
+
+    ret = NvCtrlGetGvoColorConversion(ctrl_target,
                                       initialCSCMatrix,
                                       initialCSCOffset,
                                       initialCSCScale);
-    
-    if (ret != NvCtrlSuccess) return NULL;
-    
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_GVO_OVERRIDE_HW_CSC, &override);
 
     if (ret != NvCtrlSuccess) return NULL;
 
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_GVO_CAPABILITIES, &caps);
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GVO_OVERRIDE_HW_CSC,
+                             &override);
+
+    if (ret != NvCtrlSuccess) return NULL;
+
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_GVO_CAPABILITIES, &caps);
 
     if (ret != NvCtrlSuccess) return NULL;
 
@@ -232,14 +229,14 @@ GtkWidget* ctk_gvo_csc_new(NvCtrlAttributeHandle *handle,
      * XXX setup to receive events when another NV-CONTROL client
      * changes any of the above attributes
      */
-    
-    
+
+
     /* create the object */
-    
+
     object = g_object_new(CTK_TYPE_GVO_CSC, NULL);
-    
+
     ctk_gvo_csc = CTK_GVO_CSC(object);
-    ctk_gvo_csc->handle = handle;
+    ctk_gvo_csc->ctrl_target = ctrl_target;
     ctk_gvo_csc->ctk_config = ctk_config;
     ctk_gvo_csc->ctk_event = ctk_event;
     ctk_gvo_csc->gvo_parent = gvo_parent;
@@ -262,7 +259,7 @@ GtkWidget* ctk_gvo_csc_new(NvCtrlAttributeHandle *handle,
     ctk_config_set_tooltip(ctk_config, ctk_gvo_csc->overrideButton,
                            __override_help);
     
-    g_signal_connect(GTK_OBJECT(ctk_gvo_csc->overrideButton), "toggled",
+    g_signal_connect(G_OBJECT(ctk_gvo_csc->overrideButton), "toggled",
                      G_CALLBACK(override_button_toggled), ctk_gvo_csc);
     
     frame = gtk_frame_new(NULL);
@@ -334,11 +331,11 @@ GtkWidget* ctk_gvo_csc_new(NvCtrlAttributeHandle *handle,
     
     
     ctk_gvo_csc->initializeDropDown =
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_MONOSPACE);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
 
-    ctk_config_set_tooltip
+    ctk_drop_down_menu_set_tooltip
         (ctk_config,
-         CTK_DROP_DOWN_MENU(ctk_gvo_csc->initializeDropDown)->option_menu,
+         CTK_DROP_DOWN_MENU(ctk_gvo_csc->initializeDropDown),
          __initialize_help);
     
     ctk_drop_down_menu_append_item
@@ -539,7 +536,7 @@ GtkWidget* ctk_gvo_csc_new(NvCtrlAttributeHandle *handle,
         gtk_toggle_button_set_active
             (GTK_TOGGLE_BUTTON(ctk_gvo_csc->applyImmediateButton), FALSE);
 
-        g_signal_connect(GTK_OBJECT(ctk_gvo_csc->applyImmediateButton),
+        g_signal_connect(G_OBJECT(ctk_gvo_csc->applyImmediateButton),
                          "toggled",
                          G_CALLBACK(apply_immediate_button_toggled),
                          ctk_gvo_csc);
@@ -560,7 +557,7 @@ GtkWidget* ctk_gvo_csc_new(NvCtrlAttributeHandle *handle,
     
     ctk_config_set_tooltip(ctk_config, ctk_gvo_csc->applyButton, __apply_help);
     
-    g_signal_connect(GTK_OBJECT(ctk_gvo_csc->applyButton),
+    g_signal_connect(G_OBJECT(ctk_gvo_csc->applyButton),
                      "clicked",
                      G_CALLBACK(apply_button_clicked),
                      ctk_gvo_csc);
@@ -688,6 +685,7 @@ static void override_button_toggled(GtkToggleButton *overrideButton,
                                     gpointer user_data)
 {
     CtkGvoCsc *ctk_gvo_csc = (CtkGvoCsc *) user_data;
+    CtrlTarget *ctrl_target = ctk_gvo_csc->ctrl_target;
     gboolean enabled = gtk_toggle_button_get_active(overrideButton);
     override_state_toggled(ctk_gvo_csc, enabled);
 
@@ -697,26 +695,26 @@ static void override_button_toggled(GtkToggleButton *overrideButton,
      * user applies it.  However, if override was disabled, apply that
      * immediately.
      */
-    
+
     if (enabled) {
-    
+
         if (ctk_gvo_csc->applyImmediately) {
-            NvCtrlSetAttribute(ctk_gvo_csc->handle,
+            NvCtrlSetAttribute(ctrl_target,
                                NV_CTRL_GVO_OVERRIDE_HW_CSC,
                                NV_CTRL_GVO_OVERRIDE_HW_CSC_TRUE);
         } else {
-        
+
             // make the "Apply" button hot
-        
+
             gtk_widget_set_sensitive(ctk_gvo_csc->applyButton, TRUE);
         }
 
     } else {
-        NvCtrlSetAttribute(ctk_gvo_csc->handle,
+        NvCtrlSetAttribute(ctrl_target,
                            NV_CTRL_GVO_OVERRIDE_HW_CSC,
                            NV_CTRL_GVO_OVERRIDE_HW_CSC_FALSE);
     }
-    
+
 } /* override_button_toggled() */
 
 
@@ -853,7 +851,7 @@ static void spin_button_value_changed(GtkWidget *button,
      * the data has changed, make sure the apply button is sensitive
      */
 
-    set_apply_button_senstive(ctk_gvo_csc);
+    set_apply_button_sensitive(ctk_gvo_csc);
 
     /* if we are supposed to apply immediately, send the data now */
 
@@ -969,7 +967,7 @@ static void initialize_csc_dropdown_changed(CtkDropDownMenu *menu,
      * the data has changed, make sure the apply button is sensitive
      */
 
-    set_apply_button_senstive(ctk_gvo_csc);
+    set_apply_button_sensitive(ctk_gvo_csc);
 
     /* if we are supposed to apply immediately, send the data now */
 
@@ -982,10 +980,10 @@ static void initialize_csc_dropdown_changed(CtkDropDownMenu *menu,
 
 
 /*
- * set_apply_button_senstive() - make the "Apply" button sensitive
+ * set_apply_button_sensitive() - make the "Apply" button sensitive
  */
 
-static void set_apply_button_senstive(CtkGvoCsc *ctk_gvo_csc)
+static void set_apply_button_sensitive(CtkGvoCsc *ctk_gvo_csc)
 {
     /* if data is applied immediately, then we don't */
 
@@ -993,7 +991,7 @@ static void set_apply_button_senstive(CtkGvoCsc *ctk_gvo_csc)
     
     gtk_widget_set_sensitive(ctk_gvo_csc->applyButton, TRUE);
     
-} /* set_apply_button_senstive() */
+} /* set_apply_button_sensitive() */
 
 
 
@@ -1004,12 +1002,14 @@ static void set_apply_button_senstive(CtkGvoCsc *ctk_gvo_csc)
 
 static void apply_csc_values(CtkGvoCsc *ctk_gvo_csc)
 {
-    NvCtrlSetGvoColorConversion(ctk_gvo_csc->handle,
+    CtrlTarget *ctrl_target = ctk_gvo_csc->ctrl_target;
+
+    NvCtrlSetGvoColorConversion(ctrl_target,
                                 ctk_gvo_csc->matrix,
                                 ctk_gvo_csc->offset,
                                 ctk_gvo_csc->scale);
-    
-    NvCtrlSetAttribute(ctk_gvo_csc->handle,
+
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_GVO_OVERRIDE_HW_CSC,
                        NV_CTRL_GVO_OVERRIDE_HW_CSC_TRUE);
 
@@ -1021,15 +1021,15 @@ static void apply_csc_values(CtkGvoCsc *ctk_gvo_csc)
  * GVO CSC event handler.
  */
 
-static void gvo_csc_event_received(GtkObject *object,
-                                   gpointer arg1,
+static void gvo_csc_event_received(GObject *object,
+                                   CtrlEvent *event,
                                    gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkGvoCsc *ctk_gvo_csc = CTK_GVO_CSC(user_data);
+    CtrlTarget *ctrl_target = ctk_gvo_csc->ctrl_target;
     GtkWidget *widget;
-    gint attribute = event_struct->attribute;
-    gint value = event_struct->value;
+    gint attribute;
+    gint value;
     ReturnStatus ret;    
 
     float newCSCMatrix[3][3];
@@ -1037,6 +1037,12 @@ static void gvo_csc_event_received(GtkObject *object,
     float newCSCScale[3];
     int row, column;
 
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
+    attribute = event->int_attr.attribute;
+    value = event->int_attr.value;
 
     switch (attribute) {
 
@@ -1044,7 +1050,7 @@ static void gvo_csc_event_received(GtkObject *object,
 
         /* Re-query the GVO CSC matrix */
 
-        ret = NvCtrlGetGvoColorConversion(ctk_gvo_csc->handle,
+        ret = NvCtrlGetGvoColorConversion(ctrl_target,
                                           newCSCMatrix,
                                           newCSCOffset,
                                           newCSCScale);
@@ -1120,7 +1126,7 @@ static void gvo_csc_event_received(GtkObject *object,
         break;
     }
 
-} /* gvo_csc_event_recieved() */
+}
 
 
 
@@ -1249,10 +1255,10 @@ GtkTextBuffer *ctk_gvo_csc_create_help(GtkTextTagTable *table,
                   "the SDI (GVO) color space conversion.");
 
     ctk_help_heading(b, &i, "Override default Color Space Conversion");
-    ctk_help_para(b, &i, __override_help);
+    ctk_help_para(b, &i, "%s", __override_help);
 
     ctk_help_heading(b, &i, "Initialize Color Space Conversion");
-    ctk_help_para(b, &i, __initialize_help);
+    ctk_help_para(b, &i, "%s", __initialize_help);
 
     ctk_help_heading(b, &i, "Color Space Conversion Matrix, Offset and Scale");
     ctk_help_para(b, &i, "%s  RGB color is converted to YCrCb in the "
@@ -1272,25 +1278,25 @@ GtkTextBuffer *ctk_gvo_csc_create_help(GtkTextTagTable *table,
                   "Scb = Cb Scaling factor\n"
                   "Scr = Cr Scaling factor");
     ctk_help_para(b, &i,
-                  "Ry  = Red coeficient for Y\n"
-                  "Rcb = Red coeficient for Cb\n"
-                  "Rcr = Red coeficient for Cr");
+                  "Ry  = Red coefficient for Y\n"
+                  "Rcb = Red coefficient for Cb\n"
+                  "Rcr = Red coefficient for Cr");
     ctk_help_para(b, &i,
-                  "Gy  = Green coeficient for Y\n"
-                  "Gcb = Green coeficient for Cb\n"
-                  "Gcr = Green coeficient for Cr");
+                  "Gy  = Green coefficient for Y\n"
+                  "Gcb = Green coefficient for Cb\n"
+                  "Gcr = Green coefficient for Cr");
     ctk_help_para(b, &i,
-                  "By  = Blue coeficient for Y\n"
-                  "Bcb = Blue coeficient for Cb\n"
-                  "Bcr = Blue coeficient for Cr");
+                  "By  = Blue coefficient for Y\n"
+                  "Bcb = Blue coefficient for Cb\n"
+                  "Bcr = Blue coefficient for Cr");
 
     if (ctk_gvo_csc->caps & NV_CTRL_GVO_CAPABILITIES_APPLY_CSC_IMMEDIATELY) {
         ctk_help_heading(b, &i, "Apply Changes Immediately");
-        ctk_help_para(b, &i, __apply_immediately_help);
+        ctk_help_para(b, &i, "%s", __apply_immediately_help);
     }
 
     ctk_help_heading(b, &i, "Apply");
-    ctk_help_para(b, &i, __apply_help);
+    ctk_help_para(b, &i, "%s", __apply_help);
 
     ctk_help_finish(b);
 
