@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -39,6 +34,8 @@ static void post_vblank_sync_button_toggled(CtkOpenGL *, gboolean);
 
 static void post_allow_flipping_button_toggled(CtkOpenGL *, gboolean);
 
+static void post_allow_gsync_button_toggled(CtkOpenGL *, gboolean);
+
 static void post_force_stereo_button_toggled(CtkOpenGL *, gboolean);
 
 static void post_show_sli_visual_indicator_button_toggled(CtkOpenGL *,
@@ -53,8 +50,12 @@ static void post_stereo_eyes_exchange_button_toggled(CtkOpenGL *, gboolean);
 
 static void post_aa_line_gamma_toggled(CtkOpenGL *, gboolean);
 
+static void post_use_conformant_clamping_button_toggled(CtkOpenGL *, gint);
+
 static void allow_flipping_button_toggled(GtkWidget *, gpointer);
 
+static void allow_gsync_button_toggled(GtkWidget *, gpointer);
+
 static void force_stereo_button_toggled (GtkWidget *, gpointer);
 
 static void xinerama_stereo_button_toggled (GtkWidget *, gpointer);
@@ -63,11 +64,13 @@ static void stereo_eyes_exchange_button_toggled (GtkWidget *, gpointer);
 
 static void aa_line_gamma_toggled        (GtkWidget *, gpointer);
 
+static void use_conformant_clamping_button_toggled(GtkWidget *, gpointer);
+
 static void show_sli_visual_indicator_button_toggled (GtkWidget *, gpointer);
 
 static void show_multigpu_visual_indicator_button_toggled (GtkWidget *, gpointer);
 
-static void value_changed                (GtkObject *, gpointer, gpointer);
+static void value_changed (GObject *, CtrlEvent *, gpointer);
 
 static const gchar *get_image_settings_string(gint val);
 
@@ -76,15 +79,15 @@ static gchar *format_image_settings_value(GtkScale *scale, gdouble arg1,
 
 static void post_slider_value_changed(CtkOpenGL *ctk_opengl, gint val);
 
-static void aa_line_gamma_update_received(GtkObject *object,
-                                         gpointer arg1, gpointer user_data);
+static void aa_line_gamma_update_received(GObject *object, CtrlEvent *event,
+                                          gpointer user_data);
 
 static void post_image_settings_value_changed(CtkOpenGL *ctk_opengl, gint val);
 
 static void image_settings_value_changed(GtkRange *range, gpointer user_data);
 
-static void image_settings_update_received(GtkObject *object,
-                                           gpointer arg1, gpointer user_data);
+static void image_settings_update_received(GObject *object, CtrlEvent *event,
+                                           gpointer user_data);
 
 static GtkWidget *create_slider(CtkOpenGL *ctk_opengl,
                                 GtkWidget *vbox,
@@ -147,6 +150,12 @@ static const char *__stereo_eyes_exchange_help =
 "eye image in the right eye and vice versa for stereo "
 "drawables.  This option is applied immediately.";
 
+static const char *__use_conformant_clamping_help =
+"Disabling this option causes OpenGL to replace GL_CLAMP with "
+"GL_CLAMP_TO_EDGE for borderless 2D textures.  This eliminates "
+"seams at the edges of textures in some older games such as "
+"Quake 3.";
+
 #define __SYNC_TO_VBLANK      (1 << 1)
 #define __ALLOW_FLIPPING      (1 << 2)
 #define __AA_LINE_GAMMA_VALUE (1 << 3)
@@ -158,6 +167,8 @@ static const char *__stereo_eyes_exchange_help =
 #define __SHOW_SLI_VISUAL_INDICATOR        (1 << 9)
 #define __STEREO_EYES_EXCHANGE (1 << 10)
 #define __SHOW_MULTIGPU_VISUAL_INDICATOR    (1 << 11)
+#define __CONFORMANT_CLAMPING (1 << 12)
+#define __ALLOW_GSYNC         (1 << 13)
 
 
 
@@ -178,6 +189,7 @@ GType ctk_opengl_get_type(
             sizeof (CtkOpenGL),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_opengl_type = g_type_register_static (GTK_TYPE_VBOX,
@@ -188,8 +200,9 @@ GType ctk_opengl_get_type(
 }
 
 
-GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
-                          CtkConfig *ctk_config, CtkEvent *ctk_event)
+GtkWidget* ctk_opengl_new(CtrlTarget *ctrl_target,
+                          CtkConfig *ctk_config,
+                          CtkEvent *ctk_event)
 {
     GObject *object;
     CtkOpenGL *ctk_opengl;
@@ -201,77 +214,108 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
     GtkWidget *vbox;
     GtkWidget *check_button;
     GtkWidget *scale;
-    GtkObject *adjustment;
+    GtkAdjustment *adjustment;
 
     gint sync_to_vblank;
     gint flipping_allowed;
+    gint gsync_allowed;
     gint force_stereo;
     gint xinerama_stereo;
     gint stereo_eyes_exchange;
     NVCTRLAttributeValidValuesRec image_settings_valid;
     gint image_settings_value;
     gint aa_line_gamma;
+    gint use_conformant_clamping;
     gint show_sli_visual_indicator;
     gint show_multigpu_visual_indicator;
 
     ReturnStatus ret_sync_to_vblank;
     ReturnStatus ret_flipping_allowed;
+    ReturnStatus ret_gsync_allowed;
     ReturnStatus ret_force_stereo;
     ReturnStatus ret_xinerama_stereo;
     ReturnStatus ret_stereo_eyes_exchange;
     ReturnStatus ret_image_settings;
     ReturnStatus ret_aa_line_gamma;
+    ReturnStatus ret_use_conformant_clamping;
     ReturnStatus ret_show_sli_visual_indicator;
     ReturnStatus ret_show_multigpu_visual_indicator;
 
     /* Query OpenGL settings */
 
     ret_sync_to_vblank =
-        NvCtrlGetAttribute(handle, NV_CTRL_SYNC_TO_VBLANK, &sync_to_vblank);
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_SYNC_TO_VBLANK,
+                           &sync_to_vblank);
 
     ret_flipping_allowed =
-        NvCtrlGetAttribute(handle, NV_CTRL_FLIPPING_ALLOWED,
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_FLIPPING_ALLOWED,
                            &flipping_allowed);
 
+    ret_gsync_allowed =
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_GSYNC_ALLOWED,
+                           &gsync_allowed);
+
     ret_force_stereo =
-        NvCtrlGetAttribute(handle, NV_CTRL_FORCE_STEREO, &force_stereo);
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_FORCE_STEREO,
+                           &force_stereo);
 
     ret_xinerama_stereo =
-        NvCtrlGetAttribute(handle, NV_CTRL_XINERAMA_STEREO, &xinerama_stereo);
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_XINERAMA_STEREO,
+                           &xinerama_stereo);
 
     ret_stereo_eyes_exchange =
-        NvCtrlGetAttribute(handle, NV_CTRL_STEREO_EYES_EXCHANGE,
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_STEREO_EYES_EXCHANGE,
                            &stereo_eyes_exchange);
 
-    ret_image_settings =
-        NvCtrlGetValidAttributeValues(handle, NV_CTRL_IMAGE_SETTINGS,
-                                      &image_settings_valid);
+    ret_image_settings = NvCtrlGetValidAttributeValues(ctrl_target,
+                                                       NV_CTRL_IMAGE_SETTINGS,
+                                                       &image_settings_valid);
     if ((ret_image_settings == NvCtrlSuccess) &&
         (image_settings_valid.type == ATTRIBUTE_TYPE_RANGE)) {
         ret_image_settings =
-            NvCtrlGetAttribute(handle, NV_CTRL_IMAGE_SETTINGS,
+            NvCtrlGetAttribute(ctrl_target,
+                               NV_CTRL_IMAGE_SETTINGS,
                                &image_settings_value);
     } else {
         ret_image_settings = NvCtrlError;
     }
 
-    ret_aa_line_gamma = NvCtrlGetAttribute(handle, NV_CTRL_OPENGL_AA_LINE_GAMMA,
-                                           &aa_line_gamma);
-    ret_show_sli_visual_indicator = NvCtrlGetAttribute(handle,
-                                          NV_CTRL_SHOW_SLI_VISUAL_INDICATOR,
-                                          &show_sli_visual_indicator);
-    ret_show_multigpu_visual_indicator = NvCtrlGetAttribute(handle,
-                                          NV_CTRL_SHOW_MULTIGPU_VISUAL_INDICATOR,
-                                          &show_multigpu_visual_indicator);
+    ret_aa_line_gamma =
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_OPENGL_AA_LINE_GAMMA,
+                           &aa_line_gamma);
+
+    ret_use_conformant_clamping =
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_TEXTURE_CLAMPING,
+                           &use_conformant_clamping);
+
+    ret_show_sli_visual_indicator =
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_SHOW_SLI_VISUAL_INDICATOR,
+                           &show_sli_visual_indicator);
+
+    ret_show_multigpu_visual_indicator =
+        NvCtrlGetAttribute(ctrl_target,
+                           NV_CTRL_SHOW_MULTIGPU_VISUAL_INDICATOR,
+                           &show_multigpu_visual_indicator);
 
     /* There are no OpenGL settings to change (OpenGL disabled?) */
     if ((ret_sync_to_vblank != NvCtrlSuccess) &&
         (ret_flipping_allowed != NvCtrlSuccess) &&
+        (ret_gsync_allowed != NvCtrlSuccess) &&
         (ret_force_stereo != NvCtrlSuccess) &&
         (ret_xinerama_stereo != NvCtrlSuccess) &&
         (ret_stereo_eyes_exchange != NvCtrlSuccess) &&
         (ret_image_settings != NvCtrlSuccess) &&
         (ret_aa_line_gamma != NvCtrlSuccess) &&
+        (ret_use_conformant_clamping != NvCtrlSuccess) &&
         (ret_show_sli_visual_indicator != NvCtrlSuccess) &&
         (ret_show_multigpu_visual_indicator != NvCtrlSuccess)) {
         return NULL;
@@ -280,7 +324,7 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
     object = g_object_new(CTK_TYPE_OPENGL, NULL);
 
     ctk_opengl = CTK_OPENGL(object);
-    ctk_opengl->handle = handle;
+    ctk_opengl->ctrl_target = ctrl_target;
     ctk_opengl->ctk_config = ctk_config;
     ctk_opengl->active_attributes = 0;
 
@@ -379,7 +423,41 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
         
         ctk_opengl->allow_flipping_button = check_button;
     }
-    
+
+    /*
+     * allow G-SYNC
+     *
+     * Always create the checkbox, but only show it if the attribute starts out
+     * available.
+     */
+
+    label = gtk_label_new("Allow G-SYNC");
+
+    check_button = gtk_check_button_new();
+    gtk_container_add(GTK_CONTAINER(check_button), label);
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
+                                 gsync_allowed);
+
+    gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
+
+    g_signal_connect(G_OBJECT(check_button), "toggled",
+                     G_CALLBACK(allow_gsync_button_toggled),
+                     (gpointer) ctk_opengl);
+
+    g_signal_connect(G_OBJECT(ctk_event),
+                     CTK_EVENT_NAME(NV_CTRL_GSYNC_ALLOWED),
+                     G_CALLBACK(value_changed), (gpointer) ctk_opengl);
+
+    ctk_config_set_tooltip(ctk_config, check_button,
+                           "Enabling this option allows OpenGL to flip "
+                           "using G-SYNC when possible.  This option is "
+                           "applied immediately.");
+
+    ctk_opengl->active_attributes |= __ALLOW_GSYNC;
+
+    ctk_opengl->allow_gsync_button = check_button;
+
     if (ret_force_stereo == NvCtrlSuccess) {
 
         label = gtk_label_new("Force Stereo Flipping");
@@ -476,10 +554,11 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
         gtk_container_add(GTK_CONTAINER(frame), hbox);
 
         /* create the slider */
-        adjustment = gtk_adjustment_new(image_settings_value,
-                                        image_settings_valid.u.range.min,
-                                        image_settings_valid.u.range.max,
-                                        1, 1, 0.0);
+        adjustment = GTK_ADJUSTMENT(
+                         gtk_adjustment_new(image_settings_value,
+                                            image_settings_valid.u.range.min,
+                                            image_settings_valid.u.range.max,
+                                            1, 1, 0.0));
         scale = gtk_hscale_new(GTK_ADJUSTMENT(adjustment));
         gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment), image_settings_value);
 
@@ -570,6 +649,37 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
                                      gtk_toggle_button_get_active
                                      (GTK_TOGGLE_BUTTON(check_button)));
     }
+
+    /*
+     * NV_CTRL_TEXTURE_CLAMPING
+     */
+
+    if (ret_use_conformant_clamping == NvCtrlSuccess) {
+        label = gtk_label_new("Use Conformant Texture Clamping");
+
+        check_button = gtk_check_button_new();
+        gtk_container_add(GTK_CONTAINER(check_button), label);
+
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
+                                     use_conformant_clamping);
+
+        gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
+
+        g_signal_connect(G_OBJECT(check_button), "toggled",
+                         G_CALLBACK(use_conformant_clamping_button_toggled),
+                         (gpointer) ctk_opengl);
+
+        g_signal_connect(G_OBJECT(ctk_event),
+                         CTK_EVENT_NAME(NV_CTRL_TEXTURE_CLAMPING),
+                         G_CALLBACK(value_changed), (gpointer) ctk_opengl);
+
+        ctk_config_set_tooltip(ctk_config, check_button,
+                               __use_conformant_clamping_help);
+
+        ctk_opengl->active_attributes |= __CONFORMANT_CLAMPING;
+
+        ctk_opengl->use_conformant_clamping_button = check_button;
+    }
     
     if (ret_show_sli_visual_indicator == NvCtrlSuccess) {
 
@@ -629,6 +739,14 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
 
     gtk_widget_show_all(GTK_WIDGET(object));
 
+    /*
+     * If GSYNC is not currently available, start out with the GSYNC button
+     * hidden.
+     */
+    if (ret_gsync_allowed != NvCtrlSuccess) {
+        gtk_widget_hide(GTK_WIDGET(ctk_opengl->allow_gsync_button));
+    }
+
     return GTK_WIDGET(object);
 }
 
@@ -651,6 +769,14 @@ static void post_allow_flipping_button_toggled(CtkOpenGL *ctk_opengl,
                                  enabled ? "allowed" : "not allowed");
 }
 
+static void post_allow_gsync_button_toggled(CtkOpenGL *ctk_opengl,
+                                          gboolean enabled)
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "G-SYNC %s.",
+                                 enabled ? "allowed" : "not allowed");
+}
+
 static void post_force_stereo_button_toggled(CtkOpenGL *ctk_opengl, 
                                              gboolean enabled)
 {
@@ -701,20 +827,26 @@ static void post_aa_line_gamma_toggled(CtkOpenGL *ctk_opengl,
                                  enabled ? "enabled" : "disabled");
 }
 
-
-static void vblank_sync_button_toggled(
-    GtkWidget *widget,
-    gpointer user_data
-)
+static void
+post_use_conformant_clamping_button_toggled(CtkOpenGL *ctk_opengl,
+                                               int clamping) 
 {
-    CtkOpenGL *ctk_opengl;
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "Use %sConformant OpenGL Texture Clamping",
+                                 (clamping == NV_CTRL_TEXTURE_CLAMPING_SPEC) ?
+                                 "" : "Non-");
+}
+
+static void vblank_sync_button_toggled(GtkWidget *widget,
+                                       gpointer user_data)
+{
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
-    
-    ctk_opengl = CTK_OPENGL(user_data);
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_SYNC_TO_VBLANK, enabled);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_SYNC_TO_VBLANK, enabled);
 
     post_vblank_sync_button_toggled(ctk_opengl, enabled);
 }
@@ -723,58 +855,66 @@ static void vblank_sync_button_toggled(
 static void allow_flipping_button_toggled(GtkWidget *widget,
                                           gpointer user_data)
 {
-    CtkOpenGL *ctk_opengl;
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
-    
-    ctk_opengl = CTK_OPENGL(user_data);
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-    
-    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_FLIPPING_ALLOWED, enabled);
+
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_FLIPPING_ALLOWED, enabled);
     post_allow_flipping_button_toggled(ctk_opengl, enabled);
-    
+}
+
+static void allow_gsync_button_toggled(GtkWidget *widget,
+                                     gpointer user_data)
+{
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
+    gboolean enabled;
+
+    enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_GSYNC_ALLOWED, enabled);
+    post_allow_gsync_button_toggled(ctk_opengl, enabled);
 }
 
 static void force_stereo_button_toggled(GtkWidget *widget,
                                          gpointer user_data)
 {
-    CtkOpenGL *ctk_opengl;
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
-    
-    ctk_opengl = CTK_OPENGL(user_data);
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_FORCE_STEREO, enabled);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_FORCE_STEREO, enabled);
     post_force_stereo_button_toggled(ctk_opengl, enabled);
 }
 
 static void show_sli_visual_indicator_button_toggled(GtkWidget *widget,
                                            gpointer user_data)
 {
-    CtkOpenGL *ctk_opengl;
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
 
-    ctk_opengl = CTK_OPENGL(user_data);
-
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_opengl->handle,
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_SHOW_SLI_VISUAL_INDICATOR, enabled);
-    post_show_sli_visual_indicator_button_toggled(ctk_opengl, enabled); 
+    post_show_sli_visual_indicator_button_toggled(ctk_opengl, enabled);
 }
 
 static void show_multigpu_visual_indicator_button_toggled(GtkWidget *widget,
                                                           gpointer user_data)
 {
-    CtkOpenGL *ctk_opengl;
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
 
-    ctk_opengl = CTK_OPENGL(user_data);
-
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_opengl->handle,
+    NvCtrlSetAttribute(ctrl_target,
                        NV_CTRL_SHOW_MULTIGPU_VISUAL_INDICATOR, enabled);
     post_show_multigpu_visual_indicator_button_toggled(ctk_opengl, enabled);
 }
@@ -782,47 +922,41 @@ static void show_multigpu_visual_indicator_button_toggled(GtkWidget *widget,
 static void xinerama_stereo_button_toggled(GtkWidget *widget,
                                            gpointer user_data)
 {
-    CtkOpenGL *ctk_opengl;
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
-    
-    ctk_opengl = CTK_OPENGL(user_data);
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_XINERAMA_STEREO, enabled);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_XINERAMA_STEREO, enabled);
     post_xinerama_stereo_button_toggled(ctk_opengl, enabled);
 }
 
 static void stereo_eyes_exchange_button_toggled(GtkWidget *widget,
                                                 gpointer user_data)
 {
-    CtkOpenGL *ctk_opengl;
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
-    
-    ctk_opengl = CTK_OPENGL(user_data);
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_STEREO_EYES_EXCHANGE,
-                       enabled);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_STEREO_EYES_EXCHANGE, enabled);
 
     post_stereo_eyes_exchange_button_toggled(ctk_opengl, enabled);
 }
 
-static void aa_line_gamma_toggled(
-    GtkWidget *widget,
-    gpointer user_data
-)
+static void aa_line_gamma_toggled(GtkWidget *widget,
+                                  gpointer user_data)
 {
-    CtkOpenGL *ctk_opengl;
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gboolean enabled;
     ReturnStatus ret;
-    
-    ctk_opengl = CTK_OPENGL(user_data);
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
-    ret = NvCtrlSetAttribute(ctk_opengl->handle,
+    ret = NvCtrlSetAttribute(ctrl_target,
                              NV_CTRL_OPENGL_AA_LINE_GAMMA, enabled);
 
     if (ret != NvCtrlSuccess) return;
@@ -831,69 +965,99 @@ static void aa_line_gamma_toggled(
     post_aa_line_gamma_toggled(ctk_opengl, enabled);
 }
 
+static void use_conformant_clamping_button_toggled(GtkWidget *widget,
+                                                   gpointer user_data)
+{
+    CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
+    int clamping;
+
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
+        clamping = NV_CTRL_TEXTURE_CLAMPING_SPEC;
+    } else {
+        clamping = NV_CTRL_TEXTURE_CLAMPING_EDGE;
+    }
+
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_TEXTURE_CLAMPING, clamping);
+
+    post_use_conformant_clamping_button_toggled(ctk_opengl, clamping);
+}
 
 
 /*
  * value_changed() - callback function for changed OpenGL settings.
  */
 
-static void value_changed(GtkObject *object, gpointer arg1, gpointer user_data)
+static void value_changed(GObject *object, CtrlEvent *event, gpointer user_data)
 {
-    CtkEventStruct *event_struct;
     CtkOpenGL *ctk_opengl;
     gboolean enabled;
+    gboolean check_available = FALSE;
     GtkToggleButton *button;
     GCallback func;
+    gint value;
+
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
 
-    event_struct = (CtkEventStruct *) arg1;
     ctk_opengl = CTK_OPENGL(user_data);
+    value = event->int_attr.value;
 
-    switch (event_struct->attribute) {
+    switch (event->int_attr.attribute) {
     case NV_CTRL_SYNC_TO_VBLANK:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->sync_to_vblank_button);
         func = G_CALLBACK(vblank_sync_button_toggled);
-        post_vblank_sync_button_toggled(ctk_opengl, event_struct->value);
+        post_vblank_sync_button_toggled(ctk_opengl, value);
         break;
     case NV_CTRL_FLIPPING_ALLOWED:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->allow_flipping_button);
         func = G_CALLBACK(allow_flipping_button_toggled);
-        post_allow_flipping_button_toggled(ctk_opengl, event_struct->value);
+        post_allow_flipping_button_toggled(ctk_opengl, value);
+        break;
+    case NV_CTRL_GSYNC_ALLOWED:
+        button = GTK_TOGGLE_BUTTON(ctk_opengl->allow_gsync_button);
+        func = G_CALLBACK(allow_gsync_button_toggled);
+        post_allow_gsync_button_toggled(ctk_opengl, value);
+        check_available = TRUE;
         break;
     case NV_CTRL_FORCE_STEREO:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->force_stereo_button);
         func = G_CALLBACK(force_stereo_button_toggled);
-        post_force_stereo_button_toggled(ctk_opengl, event_struct->value);
+        post_force_stereo_button_toggled(ctk_opengl, value);
         break;
     case NV_CTRL_XINERAMA_STEREO:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->xinerama_stereo_button);
         func = G_CALLBACK(xinerama_stereo_button_toggled);
-        post_xinerama_stereo_button_toggled(ctk_opengl, event_struct->value);
+        post_xinerama_stereo_button_toggled(ctk_opengl, value);
         break;
     case NV_CTRL_STEREO_EYES_EXCHANGE:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->stereo_eyes_exchange_button);
         func = G_CALLBACK(stereo_eyes_exchange_button_toggled);
-        post_stereo_eyes_exchange_button_toggled(ctk_opengl,
-                                                 event_struct->value);
+        post_stereo_eyes_exchange_button_toggled(ctk_opengl, value);
         break;
     case NV_CTRL_OPENGL_AA_LINE_GAMMA:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->aa_line_gamma_button);
         func = G_CALLBACK(aa_line_gamma_toggled);
-        post_aa_line_gamma_toggled(ctk_opengl, event_struct->value);
-        gtk_widget_set_sensitive(ctk_opengl->aa_line_gamma_scale,
-                                 event_struct->value);
+        post_aa_line_gamma_toggled(ctk_opengl, value);
+        gtk_widget_set_sensitive(ctk_opengl->aa_line_gamma_scale, value);
+        break;
+    case NV_CTRL_TEXTURE_CLAMPING:
+        button =
+            GTK_TOGGLE_BUTTON(ctk_opengl->use_conformant_clamping_button);
+        func = G_CALLBACK(use_conformant_clamping_button_toggled);
+        post_use_conformant_clamping_button_toggled(ctk_opengl, value);
         break;
     case NV_CTRL_SHOW_SLI_VISUAL_INDICATOR:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->show_sli_visual_indicator_button);
         func = G_CALLBACK(show_sli_visual_indicator_button_toggled);
-        post_show_sli_visual_indicator_button_toggled(ctk_opengl,
-                                                      event_struct->value);
+        post_show_sli_visual_indicator_button_toggled(ctk_opengl, value);
         break;
     case NV_CTRL_SHOW_MULTIGPU_VISUAL_INDICATOR:
         button =
             GTK_TOGGLE_BUTTON(ctk_opengl->show_multigpu_visual_indicator_button);
         func = G_CALLBACK(show_multigpu_visual_indicator_button_toggled);
-        post_show_multigpu_visual_indicator_button_toggled(ctk_opengl,
-                                                           event_struct->value);
+        post_show_multigpu_visual_indicator_button_toggled(ctk_opengl, value);
         break;
     default:
         return;
@@ -901,13 +1065,21 @@ static void value_changed(GtkObject *object, gpointer arg1, gpointer user_data)
     
     enabled = gtk_toggle_button_get_active(button);
 
-    if (enabled != event_struct->value) {
+    if (enabled != value) {
         
         g_signal_handlers_block_by_func(button, func, ctk_opengl);
-        gtk_toggle_button_set_active(button, event_struct->value);
+        gtk_toggle_button_set_active(button, value);
         g_signal_handlers_unblock_by_func(button, func, ctk_opengl);
     }
-    
+
+    if (check_available && event->int_attr.is_availability_changed) {
+        if (event->int_attr.availability) {
+            gtk_widget_show(GTK_WIDGET(button));
+        } else {
+            gtk_widget_hide(GTK_WIDGET(button));
+        }
+    }
+
 } /* value_changed() */
 
 
@@ -963,9 +1135,10 @@ static void post_image_settings_value_changed(CtkOpenGL *ctk_opengl, gint val)
 static void image_settings_value_changed(GtkRange *range, gpointer user_data)
 {
     CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gint val = gtk_range_get_value(range);
 
-    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_IMAGE_SETTINGS, val);
+    NvCtrlSetAttribute(ctrl_target, NV_CTRL_IMAGE_SETTINGS, val);
     post_image_settings_value_changed(ctk_opengl, val);
 
 } /* image_settings_value_changed() */
@@ -975,19 +1148,23 @@ static void image_settings_value_changed(GtkRange *range, gpointer user_data)
  * NV_CTRL_IMAGE_SETTINGS atribute is changed by another NV-CONTROL client.
  */
 
-static void image_settings_update_received(GtkObject *object,
-                                          gpointer arg1, gpointer user_data)
+static void image_settings_update_received(GObject *object,
+                                           CtrlEvent *event,
+                                           gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
     GtkRange *range = GTK_RANGE(ctk_opengl->image_settings_scale);
 
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
     g_signal_handlers_block_by_func(G_OBJECT(range),
                                     G_CALLBACK(image_settings_value_changed),
                                     (gpointer) ctk_opengl);
 
-    gtk_range_set_value(range, event_struct->value);
-    post_image_settings_value_changed(ctk_opengl, event_struct->value);
+    gtk_range_set_value(range, event->int_attr.value);
+    post_image_settings_value_changed(ctk_opengl, event->int_attr.value);
 
     g_signal_handlers_unblock_by_func(G_OBJECT(range),
                                       G_CALLBACK(image_settings_value_changed),
@@ -1019,36 +1196,46 @@ static void post_slider_value_changed(CtkOpenGL *ctk_opengl, gint val)
 static void slider_changed(GtkAdjustment *adjustment, gpointer user_data)
 {
     CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
     gint attribute, value;
+
     user_data = g_object_get_data(G_OBJECT(adjustment), "opengl_attribute");
     attribute = GPOINTER_TO_INT(user_data);
-    value = (gint) adjustment->value;
-    NvCtrlSetAttribute(ctk_opengl->handle, attribute, value);
+    value = (gint) gtk_adjustment_get_value(adjustment);
+
+    NvCtrlSetAttribute(ctrl_target, attribute, value);
+
     post_slider_value_changed(ctk_opengl, value);
+
 } /* slider_changed() */
 
 
 /*
  * aa_line_gamma_update_received() - this function is called when the
- * NV_CTRL_OPENGL_AA_LINE_GAMMA_VALUE atribute is changed by another NV-CONTROL
+ * NV_CTRL_OPENGL_AA_LINE_GAMMA_VALUE attribute is changed by another NV-CONTROL
  * client.
  */
 
-static void aa_line_gamma_update_received(GtkObject *object,
-                                         gpointer arg1, gpointer user_data)
+static void aa_line_gamma_update_received(GObject *object,
+                                          CtrlEvent *event,
+                                          gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkOpenGL *ctk_opengl = CTK_OPENGL(user_data);
     CtkScale *scale = CTK_SCALE(ctk_opengl->aa_line_gamma_scale);
     GtkAdjustment *adjustment;
+
+    if (event->type != CTRL_EVENT_TYPE_INTEGER_ATTRIBUTE) {
+        return;
+    }
+
     adjustment = GTK_ADJUSTMENT(scale->gtk_adjustment);
     g_signal_handlers_block_by_func(G_OBJECT(adjustment),
                                     G_CALLBACK(slider_changed),
                                     (gpointer) ctk_opengl);
 
     gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),
-                             (gint)event_struct->value);
-    post_slider_value_changed(ctk_opengl, event_struct->value);
+                             (gint)event->int_attr.value);
+    post_slider_value_changed(ctk_opengl, event->int_attr.value);
 
     g_signal_handlers_unblock_by_func(G_OBJECT(adjustment),
                                       G_CALLBACK(slider_changed),
@@ -1065,18 +1252,19 @@ static GtkWidget *create_slider(CtkOpenGL *ctk_opengl,
                                 gint attribute,
                                 unsigned int bit)
 {
-    GtkObject *adjustment;
+    CtrlTarget *ctrl_target = ctk_opengl->ctrl_target;
+    GtkAdjustment *adjustment;
     GtkWidget *scale, *widget;
     gint min, max, val, step_incr, page_incr;
     NVCTRLAttributeValidValuesRec range;
     ReturnStatus ret;
     /* get the attribute value */
 
-    ret = NvCtrlGetAttribute(ctk_opengl->handle, attribute, &val);
+    ret = NvCtrlGetAttribute(ctrl_target, attribute, &val);
     if (ret != NvCtrlSuccess) return NULL;
     /* get the range for the attribute */
 
-    NvCtrlGetValidAttributeValues(ctk_opengl->handle, attribute, &range);
+    NvCtrlGetValidAttributeValues(ctrl_target, attribute, &range);
 
     if (range.type != ATTRIBUTE_TYPE_RANGE) return NULL;
     min = range.u.range.min;
@@ -1088,8 +1276,8 @@ static GtkWidget *create_slider(CtkOpenGL *ctk_opengl,
     if (page_incr <= 0) page_incr = 1;
 
     /* create the slider */
-    adjustment = gtk_adjustment_new(val, min, max,
-                                    step_incr, page_incr, 0.0);
+    adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(val, min, max,
+                                                   step_incr, page_incr, 0.0));
 
     g_object_set_data(G_OBJECT(adjustment), "opengl_attribute",
                       GINT_TO_POINTER(attribute));
@@ -1126,7 +1314,7 @@ GtkTextBuffer *ctk_opengl_create_help(GtkTextTagTable *table,
 
     if (ctk_opengl->active_attributes & __SYNC_TO_VBLANK) {
         ctk_help_heading(b, &i, "Sync to VBlank");
-        ctk_help_para(b, &i, __sync_to_vblank_help);
+        ctk_help_para(b, &i, "%s", __sync_to_vblank_help);
     }
 
     if (ctk_opengl->active_attributes & __ALLOW_FLIPPING) {
@@ -1146,19 +1334,38 @@ GtkTextBuffer *ctk_opengl_create_help(GtkTextTagTable *table,
                       "after the option is set.");
     }
 
+    if (ctk_opengl->active_attributes & __ALLOW_GSYNC) {
+        ctk_help_heading(b, &i, "Allow G-SYNC");
+        ctk_help_para(b, &i, "Enabling this option allows OpenGL to use G-SYNC "
+                      "when available.  G-SYNC is a technology that allows a "
+                      "monitor to delay updating the screen until the GPU is "
+                      "ready to display a new frame.  Without G-SYNC, the GPU "
+                      "waits for the display to be ready to accept a new frame "
+                      "instead.");
+
+        ctk_help_para(b, &i, "Note that this option is applied immediately, "
+                      "unlike most other OpenGL options which are only "
+                      "applied to OpenGL applications that are started "
+                      "after the option is set.");
+
+        ctk_help_para(b, &i, "This option can be overridden on a "
+                      "per-application basis using the GLGSYNCAllowed "
+                      "application profile key.");
+    }
+
     if (ctk_opengl->active_attributes & __FORCE_STEREO) {
         ctk_help_heading(b, &i, "Force Stereo Flipping");
-        ctk_help_para(b, &i, __force_stereo_help);
+        ctk_help_para(b, &i, "%s", __force_stereo_help);
     }
     
     if (ctk_opengl->active_attributes & __XINERAMA_STEREO) {
         ctk_help_heading(b, &i, "Allow Xinerama Stereo Flipping");
-        ctk_help_para(b, &i, __xinerama_stereo_help);
+        ctk_help_para(b, &i, "%s", __xinerama_stereo_help);
     }
     
     if (ctk_opengl->active_attributes & __STEREO_EYES_EXCHANGE) {
         ctk_help_heading(b, &i, "Exchange Stereo Eyes");
-        ctk_help_para(b, &i, __stereo_eyes_exchange_help);
+        ctk_help_para(b, &i, "%s", __stereo_eyes_exchange_help);
     }
     
     if (ctk_opengl->active_attributes & __IMAGE_SETTINGS) {
@@ -1197,13 +1404,18 @@ GtkTextBuffer *ctk_opengl_create_help(GtkTextTagTable *table,
     if (ctk_opengl->active_attributes & __AA_LINE_GAMMA) {
         ctk_help_heading(b, &i, "Enable gamma correction for "
                          "antialiased lines");
-        ctk_help_para(b, &i, __aa_line_gamma_checkbox_help );
+        ctk_help_para(b, &i, "%s", __aa_line_gamma_checkbox_help );
     }
 
     if (ctk_opengl->active_attributes & __AA_LINE_GAMMA_VALUE) {
         ctk_help_heading(b, &i, "Set gamma correction for "
                          "antialiased lines");
-        ctk_help_para(b, &i, __aa_line_gamma_slider_help);
+        ctk_help_para(b, &i, "%s", __aa_line_gamma_slider_help);
+    }
+
+    if (ctk_opengl->active_attributes & __CONFORMANT_CLAMPING) {
+        ctk_help_heading(b, &i, "Use Conformant Texture Clamping");
+        ctk_help_para(b, &i, "%s", __use_conformant_clamping_help);
     }
 
     if (ctk_opengl->active_attributes & __SHOW_SLI_VISUAL_INDICATOR) {
