@@ -34,43 +34,13 @@
 #include "ctkutils.h"
 #include "ctkbanner.h"
 
-static const _CtkStereoMode stereoMode[] = {
-    { NV_CTRL_STEREO_OFF,                          "Stereo Disabled" },
-    { NV_CTRL_STEREO_DDC,                          "DDC Stereo" },
-    { NV_CTRL_STEREO_BLUELINE,                     "Blueline Stereo" },
-    { NV_CTRL_STEREO_DIN,                          "Onboard DIN Stereo" },
-    { NV_CTRL_STEREO_PASSIVE_EYE_PER_DPY,          "Passive One-Eye-per-Display Stereo" },
-    { NV_CTRL_STEREO_VERTICAL_INTERLACED,          "Vertical Interlaced Stereo" },
-    { NV_CTRL_STEREO_COLOR_INTERLACED,             "Color Interleaved Stereo" },
-    { NV_CTRL_STEREO_HORIZONTAL_INTERLACED,        "Horizontal Interlaced Stereo" },
-    { NV_CTRL_STEREO_CHECKERBOARD_PATTERN,         "Checkerboard Pattern Stereo" },
-    { NV_CTRL_STEREO_INVERSE_CHECKERBOARD_PATTERN, "Inverse Checkerboard Stereo" },
-    { NV_CTRL_STEREO_3D_VISION,                    "NVIDIA 3D Vision Stereo" },
-    { NV_CTRL_STEREO_3D_VISION_PRO,                "NVIDIA 3D Vision Pro Stereo" },
-    { NV_CTRL_STEREO_HDMI_3D,                      "HDMI 3D Stereo" },
-};
-
 void ctk_screen_event_handler(GtkWidget *widget,
-                              XRRScreenChangeNotifyEvent *ev,
+                              CtrlEvent *event,
                               gpointer data);
 
-static void associated_displays_received(GtkObject *object, gpointer arg1,
-                                         gpointer user_data);
-
-static void info_update_gpu_error(GtkObject *object, gpointer arg1,
+static void info_update_gpu_error(GObject *object, gpointer arg1,
                                       gpointer user_data);
 
-static const char *get_stereo_mode_string(int stereo_mode)
-{
-    int i;
-    for (i = 0; i < ARRAY_LEN(stereoMode); i++) {
-        if (stereoMode[i].stereo_mode == stereo_mode) {
-            return stereoMode[i].name;
-        }
-    }
-    return "Unknown";
-}
-
 GType ctk_screen_get_type(
     void
 )
@@ -102,11 +72,11 @@ GType ctk_screen_get_type(
 
 
 /* Generates a list of display devices for the logical X screen
- * given as "handle".
+ * given as CtrlTarget.
  */
-static gchar *make_display_device_list(NvCtrlAttributeHandle *handle)
+static gchar *make_display_device_list(CtrlTarget *ctrl_target)
 {
-    return create_display_name_list_string(handle,
+    return create_display_name_list_string(ctrl_target,
                                            NV_CTRL_BINARY_DATA_DISPLAYS_ENABLED_ON_XSCREEN);
 } /* make_display_device_list() */
 
@@ -151,8 +121,7 @@ in this Software without prior written authorization from The Open Group.
  *
  */
 
-GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
-                          CtkEvent *ctk_event)
+GtkWidget* ctk_screen_new(CtrlTarget *ctrl_target, CtkEvent *ctk_event)
 {
     GObject *object;
     CtkScreen *ctk_screen;
@@ -187,15 +156,15 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
      * get the data that we will display below
      */
 
-    screen_number = g_strdup_printf("%d", NvCtrlGetTargetId(handle));
-    
-    display_name = NvCtrlGetDisplayName(handle);
-    
+    screen_number = g_strdup_printf("%d", NvCtrlGetTargetId(ctrl_target));
+
+    display_name = NvCtrlGetDisplayName(ctrl_target);
+
     dimensions = g_strdup_printf("%dx%d pixels (%dx%d millimeters)",
-                                 NvCtrlGetScreenWidth(handle),
-                                 NvCtrlGetScreenHeight(handle),
-                                 NvCtrlGetScreenWidthMM(handle),
-                                 NvCtrlGetScreenHeightMM(handle));
+                                 NvCtrlGetScreenWidth(ctrl_target),
+                                 NvCtrlGetScreenHeight(ctrl_target),
+                                 NvCtrlGetScreenWidthMM(ctrl_target),
+                                 NvCtrlGetScreenHeightMM(ctrl_target));
     
     /*
      * there are 2.54 centimeters to an inch; so there are 25.4 millimeters.
@@ -205,40 +174,42 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
      *         = N * 25.4 pixels / M inch
      */
     
-    xres = (((double) NvCtrlGetScreenWidth(handle)) * 25.4) / 
-        ((double) NvCtrlGetScreenWidthMM(handle));
+    xres = (((double) NvCtrlGetScreenWidth(ctrl_target)) * 25.4) / 
+        ((double) NvCtrlGetScreenWidthMM(ctrl_target));
 
-    yres = (((double) NvCtrlGetScreenHeight(handle)) * 25.4) / 
-        ((double) NvCtrlGetScreenHeightMM(handle));
+    yres = (((double) NvCtrlGetScreenHeight(ctrl_target)) * 25.4) / 
+        ((double) NvCtrlGetScreenHeightMM(ctrl_target));
 
     resolution = g_strdup_printf("%dx%d dots per inch", 
                                  (int) (xres + 0.5),
                                  (int) (yres + 0.5));
     
-    depth = g_strdup_printf("%d", NvCtrlGetScreenPlanes(handle));
+    depth = g_strdup_printf("%d", NvCtrlGetScreenPlanes(ctrl_target));
     
     /* get the list of GPUs driving this (logical) X screen */
 
     gpus = NULL;
-    ret = NvCtrlGetBinaryAttribute(handle,
+    ret = NvCtrlGetBinaryAttribute(ctrl_target,
                                    0,
                                    NV_CTRL_BINARY_DATA_GPUS_USED_BY_LOGICAL_XSCREEN,
                                    (unsigned char **)(&pData),
                                    &len);
     if (ret == NvCtrlSuccess) {
         for (i = 1; i <= pData[0]; i++) {
+            CtrlTarget *gpu_target;
             gchar *tmp_str;
             gchar *gpu_name;
-            Bool valid;
 
-            valid =
-                XNVCTRLQueryTargetStringAttribute(NvCtrlGetDisplayPtr(handle),
-                                                  NV_CTRL_TARGET_TYPE_GPU,
-                                                  pData[i],
-                                                  0,
-                                                  NV_CTRL_STRING_PRODUCT_NAME,
-                                                  &gpu_name);
-            if (!valid) {
+            gpu_target = NvCtrlGetTarget(ctrl_target->system,
+                                         GPU_TARGET, pData[i]);
+            if (gpu_target == NULL) {
+                continue;
+            }
+
+            ret = NvCtrlGetStringAttribute(gpu_target,
+                                           NV_CTRL_STRING_PRODUCT_NAME,
+                                           &gpu_name);
+            if (ret != NvCtrlSuccess) {
                 gpu_name = "Unknown";
             }
             if (gpus) {
@@ -247,8 +218,8 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
             } else {
                 tmp_str = g_strdup_printf("%s (GPU %d)", gpu_name, pData[i]);
             }
-            if (valid) {
-                XFree(gpu_name);
+            if (ret == NvCtrlSuccess) {
+                free(gpu_name);
             }
             g_free(gpus);
             gpus = tmp_str;
@@ -256,17 +227,17 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
         if (!gpus) {
             gpus = g_strdup("None");
         }
-        XFree(pData);
+        free(pData);
     }
 
     /* get the list of Display Devices displaying this X screen */
 
-    displays = make_display_device_list(handle);
+    displays = make_display_device_list(ctrl_target);
 
     /* get the number of gpu errors occurred */
 
     gpu_errors = 0;
-    ret = NvCtrlGetAttribute(handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_NUM_GPU_ERRORS_RECOVERED,
                              (int *)&gpu_errors);
 
@@ -277,12 +248,12 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
     object = g_object_new(CTK_TYPE_SCREEN, NULL);
     ctk_screen = CTK_SCREEN(object);
 
-    /* cache the attribute handle */
+    /* cache the control target */
 
-    ctk_screen->handle = handle;
+    ctk_screen->ctrl_target = ctrl_target;
 
     /* get the stereo mode set for this X screen */
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_STEREO, (int *)&stereo_mode);
+    ret = NvCtrlGetAttribute(ctrl_target, NV_CTRL_STEREO, (int *)&stereo_mode);
     ctk_screen->stereo_available = (ret == NvCtrlSuccess);
 
     /* set container properties of the object */
@@ -339,7 +310,7 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
         add_table_row(table, 19, 0, 0, "Recovered GPU Errors:", 0, 0, tmp);
     if (ctk_screen->stereo_available) {
         add_table_row(table, 20, 0, 0, "Stereo Mode:", 0, 0,
-                      get_stereo_mode_string(stereo_mode));
+                      NvCtrlGetStereoModeName(stereo_mode));
     }
 
     g_free(screen_number);
@@ -349,12 +320,6 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
     g_free(depth);
     g_free(gpus);
     g_free(displays);
-    
-    /* Handle updates to the list of associated display devices */
-    g_signal_connect(G_OBJECT(ctk_event),
-                     CTK_EVENT_NAME(NV_CTRL_ASSOCIATED_DISPLAY_DEVICES),
-                     G_CALLBACK(associated_displays_received),
-                     (gpointer) ctk_screen);
 
     /* Setup widget to handle XRRScreenChangeNotify events */
     g_signal_connect(G_OBJECT(ctk_event), "CTK_EVENT_RRScreenChangeNotify",
@@ -446,14 +411,21 @@ GtkTextBuffer *ctk_screen_create_help(GtkTextTagTable *table,
  * with the new screen information.
  */
 void ctk_screen_event_handler(GtkWidget *widget,
-                              XRRScreenChangeNotifyEvent *ev,
+                              CtrlEvent *event,
                               gpointer data)
 {
     CtkScreen *ctk_screen = (CtkScreen *) data;
+    gchar *dimensions;
 
-    gchar *dimensions =  g_strdup_printf("%dx%d pixels (%dx%d millimeters)",
-                                         ev->width, ev->height,
-                                         ev->mwidth, ev->mheight);
+    if (event->type != CTRL_EVENT_TYPE_SCREEN_CHANGE) {
+        return;
+    }
+
+    dimensions =  g_strdup_printf("%dx%d pixels (%dx%d millimeters)",
+                                  event->screen_change.width,
+                                  event->screen_change.height,
+                                  event->screen_change.mwidth,
+                                  event->screen_change.mheight);
 
     gtk_label_set_text(GTK_LABEL(ctk_screen->dimensions),
                        dimensions);
@@ -464,31 +436,12 @@ void ctk_screen_event_handler(GtkWidget *widget,
 
 
 
-/*
- * When the list of associated displays on this screen changes, we should
- * update the display device list shown on the page.
- */
-static void associated_displays_received(GtkObject *object, gpointer arg1,
-                                         gpointer user_data)
-{
-    CtkScreen *ctk_object = CTK_SCREEN(user_data);
-    gchar *str;
-
-    str = make_display_device_list(ctk_object->handle);
-
-    gtk_label_set_text(GTK_LABEL(ctk_object->displays), str);
-
-    g_free(str);
-
-} /* associated_displays_received() */
-
-
 /*
  * When the number of gpu errors occurred changes,
  * update the count showed on the page.
  */
 
-static void info_update_gpu_error(GtkObject *object, gpointer arg1,
+static void info_update_gpu_error(GObject *object, gpointer arg1,
                                          gpointer data)
 {
     CtkScreen *ctk_screen = (CtkScreen *) data;
@@ -497,7 +450,7 @@ static void info_update_gpu_error(GtkObject *object, gpointer arg1,
     char tmp[16];
 
     /* get the number of gpu errors occurred */
-    ret = NvCtrlGetAttribute(ctk_screen->handle,
+    ret = NvCtrlGetAttribute(ctk_screen->ctrl_target,
                              NV_CTRL_NUM_GPU_ERRORS_RECOVERED,
                              (int *)&gpu_errors);
     if (ret == NvCtrlSuccess) {
